#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *
from .LogMessageFormatter import *





#
# This logger writes log data to a log file.
#
class SimpleFileLogger(AbstractLogger):



	def __init__(self, filePath, idCounter = None, parentID = None, indentationLevel = 0,
		bAppendToExistingFile = True, bFlushAfterEveryLogMessage = True, fileMode = 0o0600, logMsgFormatter = None):
		super().__init__(idCounter)
		self._indentationLevel = indentationLevel
		if parentID is None:
			parentID = self._idCounter.next()
		self._parentLogEntryID = parentID
		self.__logMsgFormatter = DEFAULT_LOG_MESSAGE_FORMATTER if logMsgFormatter is None else logMsgFormatter
		assert isinstance(self.__logMsgFormatter, AbstractLogMessageFormatter)

		self.__filePath = filePath
		self.__bAppendToExistingFile = bAppendToExistingFile
		self.__bFlushAfterEveryLogMessage = bFlushAfterEveryLogMessage
		if not bAppendToExistingFile:
			if os.path.isfile(filePath):
				os.unlink(filePath)
		self.__f = open(filePath, "a")
		if fileMode is not None:
			os.chmod(filePath, fileMode)
		self.__fileMode = fileMode
	#



	@staticmethod
	def create(filePath, bAppendToExistingFile = True, bFlushAfterEveryLogMessage = True, fileMode = 0o0600, logMsgFormatter = None):
		assert isinstance(filePath, str)
		assert isinstance(bAppendToExistingFile, bool)
		assert isinstance(bFlushAfterEveryLogMessage, bool)
		assert isinstance(fileMode, int)
		if logMsgFormatter != None:
			assert isinstance(logMsgFormatter, AbstractLogMessageFormatter)
		return SimpleFileLogger(filePath, None, None, 0, bAppendToExistingFile, bFlushAfterEveryLogMessage, fileMode, logMsgFormatter)
	#



	def _logi(self, logEntryStruct, bNeedsIndentationLevelAdaption):
		if self.__f.closed:
			raise Exception("Logger already closed.")

		if bNeedsIndentationLevelAdaption:
			logEntryStruct = list(logEntryStruct)
			logEntryStruct[2] = self._indentationLevel
		lineOrLines = self.__logMsgFormatter.format(logEntryStruct)
		if isinstance(lineOrLines, str):
			self.__f.write(lineOrLines + "\n")
		else:
			for line in lineOrLines:
				self.__f.write(line + "\n")
		if self.__bFlushAfterEveryLogMessage:
			self.__f.flush()
	#



	# TODO: provide a more efficient implementation. currently <c>_logi()</c> will be called through <c>_logiAll()</c> so that not single but multiple tests are performed if the logger is already closed and multiple flushes are performed.
	def _logiAll(self, logEntryStructList, bNeedsIndentationLevelAdaption):
		if self.__f.closed:
			raise Exception("Logger already closed.")

		#for logEntryStruct in logEntryStructList:
		#	lineOrLines = self.__logMsgFormatter.format(logEntryStruct)
		#	if isinstance(lineOrLines, str):
		#		self.__f.write(lineOrLines + "\n")
		#	else:
		#		for line in lineOrLines:
		#			self.__f.write(line + "\n")

		super()._logiAll(logEntryStructList, bNeedsIndentationLevelAdaption)

		#if self.__bFlushAfterEveryLogMessage:
		#	self.__f.flush()
	#



	def _descend(self, logEntryStruct):
		nextID = logEntryStruct[1]
		return SimpleFileLogger(self.__filePath, self._idCounter, nextID, self._indentationLevel + 1,
			self.__bAppendToExistingFile, self.__bFlushAfterEveryLogMessage, self.__fileMode, self.__logMsgFormatter)
	#



	def close(self):
		if not self.__f.closed:
			self.__f.flush()
			self.__f.close()
	#









#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *





#
# This object represents a string list buffer.
#
class StringListBuffer(object):

	#
	# Constructor method.
	#
	def __init__(self):
		self.__list = []



	#
	# Write one or more text lines to the log file.
	#
	# @param		str prefix				If not <c>None</c> this text is prepended to the log file.
	# @param		mixed lineOrLines		A single text string or a list of text lines to write. Don't specify <c>None</c> here.
	#
	def print(self, prefix, lineOrLines):
		if self.__list is None:
			raise Exception("File already closed!")

		if isinstance(lineOrLines, str):
			if prefix is None:
				self.__list.append(lineOrLines)
			else:
				self.__list.append(prefix + lineOrLines)
		else:
			if prefix is None:
				for line in lineOrLines:
					self.__list.append(line)
			else:
				for line in lineOrLines:
					self.__list.append(prefix + line)



	#
	#
	#
	def getDataAsList(self):
		return self.__list



	#
	#
	#
	def getDataAsStr(self):
		ret = ""
		for line in self.__list:
			ret += line + "\n"
		return ret



	#
	# Close the file.
	#
	def close(self):
		self.__list = None



	#
	# Checks if the file is closed.
	#
	# @return	bool		Returns <c>true</c> if the file is already closed.
	#
	@property
	def isClosed(self):
		return self.__list is None

	#
	# Checks if the file is closed.
	#
	# @return	bool		Returns <c>true</c> if the file is already closed.
	#
	@property
	def closed(self):
		return self.__list is None

	#
	# Removes all data.
	#
	def clear(self):
		if self.__list is None:
			return
		self.__list = []

#












#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *





#
# This logger serves as a filter. Logging events are passed on to other loggers if they are within the
# accepting range of the filter.
#
class FilterLogger(AbstractLogger):


	def __init__(self, logger, minLogLevel):
		super().__init__(None)
		self.__logger = logger
		self.__minLogLevel = minLogLevel
	#



	@staticmethod
	def create(logger, minLogLevel = EnumLogLevel.WARNING):
		assert isinstance(logger, AbstractLogger)

		return FilterLogger(logger, [ int(minLogLevel) ])
	#



	def _log(self, timeStamp, logLevel, textOrException):
		if int(logLevel) >= self.__minLogLevel[0]:
			self.__logger._log(timeStamp, logLevel, textOrException)



	@property
	def minLogLevel(self):
		return self.__minLogLevel[0]



	def setMinLogLevel(self, minLogLevel):
		self.__minLogLevel[0] = minLogLevel



	def _descend(self, logEntryStruct):
		return FilterLogger(self.__logger._descend(logEntryStruct), self.__minLogLevel)



	def clear(self):
		self.__logger.clear()











#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *
from .AbstractLogMessageFormatter import *
from .LogMessageFormatter import *
from .RollOverLogFile import *





#
# This logger writes log data to a log file. Log file rotation is supported.
#
class FileLogger(AbstractLogger):


	def __init__(self, idCounter = None, parentID = None, indentationLevel = None, f = None, prefix = None, logMsgFormatter = None):
		super().__init__(idCounter)
		if parentID is None:
			parentID = self._idCounter.next()
		else:
			assert isinstance(parentID, int)
		assert isinstance(f, RollOverLogFile)
		assert isinstance(indentationLevel, int)
		if prefix != None:
			assert isinstance(prefix, str)
		self.__logMsgFormatter = DEFAULT_LOG_MESSAGE_FORMATTER if logMsgFormatter is None else logMsgFormatter
		assert isinstance(self.__logMsgFormatter, AbstractLogMessageFormatter)

		self.__f = f
		self._indentationLevel = indentationLevel
		self.__prefix = prefix
	#



	def _logi(self, logEntryStruct, bNeedsIndentationLevelAdaption):
		if self.__f.isClosed:
			raise Exception("Logger already closed.")

		if bNeedsIndentationLevelAdaption:
			logEntryStruct = list(logEntryStruct)
			logEntryStruct[2] = self._indentationLevel
		lineOrLines = self.__logMsgFormatter.format(logEntryStruct)
		self.__f.print(None, lineOrLines)
	#



	# TODO: provide a more efficient implementation. currently <c>_logi()</c> will be called through <c>_logiAll()</c> so that not single but multiple tests are performed if the logger is already closed.
	def _logiAll(self, logEntryStructList, bNeedsIndentationLevelAdaption):
		if self.__f.closed:
			raise Exception("Logger already closed.")

		super()._logiAll(logEntryStructList, bNeedsIndentationLevelAdaption)
	#



	def _descend(self, logEntryStruct):
		nextID = logEntryStruct[1]
		return FileLogger(self._idCounter, nextID, self._indentationLevel + 1,
			self.__f, self.__prefix, self.__logMsgFormatter)

	#



	#
	# Create a new instance of this logger.
	#
	# @param		str filePath									The path of the file to write the log data to. If <c>rollOverMode</c> is not <c>None</c>
	#																you need to use the following shortcuts within the file path to be filled in with date and
	#																time values:
	#																* %Y - year (f.e. 2017with date and
	#																* %m - month (f.e. 08)
	#																* %d - day (f.e. 02)
	#																* %H - hour (f.e. 03)
	#																* %M - minute (f.e. 49)
	#																* %S - second (f.e. 07)
	#																All values used in the pattern elements are padded with zeros.
	# @param		str rollOverMode								Either specify <c>None</c> if no roll-over should be used. Otherwise
	#																specify either "second", "minute", "hour", "day" or "month".
	# @param		bool bAppendToExistingFile						If <c>True</c> a possibly already existing file will be reused, data will
	#																be appended. If <c>False</c> the file will be recreated if it already exists.
	# @param		bool bFlushAfterEveryWrite						If <c>True</c> perform a flush after every write.
	# @param		int fileMode									If not <c>None</c> <c>chmod()</c> will be called with these file modes specified here.
	# @param		AbstractLogMessageFormatter logMsgFormatter		A log message formatter. If <c>None</c> is specified the default one is used.
	#
	@staticmethod
	def create(filePath, rollOver, bAppendToExistingFile = True, bFlushAfterEveryLogMessage = True, fileMode = 0o600,
		logMsgFormatter = None):
		assert isinstance(filePath, str)
		if rollOver is not None:
			assert rollOver in ["second", "minute", "hour", "day", "month", "none"]
			if rollOver == "none":
				rollOver = None
		assert isinstance(bAppendToExistingFile, bool)
		assert isinstance(bFlushAfterEveryLogMessage, bool)
		assert isinstance(fileMode, int)

		logFile = RollOverLogFile(filePath, rollOver, bAppendToExistingFile, bFlushAfterEveryLogMessage, fileMode)
		return FileLogger(None, None, 0, logFile, None, logMsgFormatter)
	#



	def close(self):
		self.__f.close()
	#



	def closed(self):
		return self.__f.isClosed
	#



	def isClosed(self):
		return self.__f.isClosed
	#



#











#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *





#
# This logger will broadcast log messages to additional loggers.
#
class NamedMulticastLogger(AbstractLogger):



	def __init__(self, loggerMap = None, idCounter = None, indentationLevel = 0, parentLogEntryID = 0):
		super().__init__(idCounter)
		self._indentationLevel = indentationLevel
		self._parentLogEntryID = parentLogEntryID

		if loggerMap is not None:
			assert isinstance(loggerMap, dict)
			self.__loggerMap = loggerMap
		else:
			self.__loggerMap = {}
	#



	@staticmethod
	def create(**kwargs):
		return NamedMulticastLogger(loggerMap = kwargs)
	#



	def addLogger(self, loggerName, logger):
		assert isinstance(loggerName, str)
		assert isinstance(logger, AbstractLogger)
		if self.__loggerMap.get(loggerName, None) is not None:
			del self.__loggerMap[loggerName]
		self.__loggerMap[loggerName] = logger
	#



	def removeLogger(self, loggerName):
		assert isinstance(loggerName, str)
		if self.__loggerMap.get(loggerName, None) is not None:
			del self.__loggerMap[loggerName]
	#



	def removeAllLoggers(self):
		self.__loggerMap = {}
	#



	def _logi(self, logEntryStruct, bNeedsIndentationLevelAdaption):
		for logger in self.__loggerMap.values():
			logger._logi(logEntryStruct, True)
	#



	def _descend(self, logEntryStruct):
		nextID = logEntryStruct[1]
		newMap = {}
		for loggerName in self.__loggerMap:
			logger = self.__loggerMap[loggerName]
			newMap[loggerName] = logger._descend(logEntryStruct)
		return NamedMulticastLogger(newMap, self._idCounter, self._indentationLevel + 1, nextID)
	#



	def clear(self):
		for logger in self.__loggerMap.values():
			logger.clear()
	#



	def close(self):
		for logger in self.__loggerList:
			logger.close()
	#









#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *





#
# This logger will broadcast log messages to additional loggers.
#
class MulticastLogger(AbstractLogger):



	def __init__(self, idCounter = None, loggerList = None, indentationLevel = 0, parentLogEntryID = 0):
		super().__init__(idCounter)
		self._indentationLevel = indentationLevel
		self._parentLogEntryID = parentLogEntryID

		self.__loggerList = []
		if loggerList is not None:
			if isinstance(loggerList, AbstractLogger):
				self.__loggerList.append(loggerList)
			elif isinstance(loggerList, (tuple, list)):
				for item in loggerList:
					if isinstance(item, AbstractLogger):
						self.__loggerList.append(item)
					else:
						raise Exception("Invalid object found in logger list: " + str(type(item)))
			else:
				raise Exception("Invalid logger list: " + str(type(loggerList)))

	#



	@staticmethod
	def create(*argv):
		return MulticastLogger(loggerList = argv)

	#



	def addLogger(self, logger):
		assert isinstance(logger, AbstractLogger)
		self.__loggerList.append(logger)

	#



	def removeLogger(self, logger):
		assert isinstance(logger, AbstractLogger)
		self.__loggerList.remove(logger)

	#



	def removeAllLoggers(self):
		self.__loggerList = []

	#



	def _logi(self, logEntryStruct, bNeedsIndentationLevelAdaption):
		for logger in self.__loggerList:
			logger._logi(logEntryStruct, True)

	#



	def _descend(self, logEntryStruct):
		nextID = logEntryStruct[1]
		newList = []
		for logger in self.__loggerList:
			newList.append(logger._descend(logEntryStruct))
		return MulticastLogger(self._idCounter, newList, self._indentationLevel + 1, nextID)

	#



	def clear(self):
		for logger in self.__loggerList:
			logger.clear()

	#



	def __str__(self):
		return "<MulticastLogger(" + hex(id(self)) + ", " + str(self.__loggerList) + ")>"

	#



	def __repr__(self):
		return "<MulticastLogger(" + hex(id(self)) + ", " + str(self.__loggerList) + ")>"

	#



	def close(self):
		for logger in self.__loggerList:
			logger.close()

	#







#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os

from enum import Enum




class EnumLogLevel(Enum):
	TRACE = 10, 'TRACE'
	DEBUG = 20, 'DEBUG'
	NOTICE = 30, 'NOTICE'
	INFO = 40, 'INFO'
	STDOUT = 41, 'STDOUT'
	SUCCESS = 50, 'SUCCESS'
	WARNING = 60, 'WARNING'
	ERROR = 70, 'ERROR'
	STDERR = 71, 'STDERR'
	EXCEPTION = 80, 'EXCEPTION'

	def __new__(cls, value, name):
		member = object.__new__(cls)
		member._value_ = value
		member.fullname = name
		return member

	def __int__(self):
		return self._value_

	def __str__(self):
		return self.fullname

	def toJSON(self):
		return self._value_

	#
	# This method converts a string or integer representing a log level back to an enumeration instance.
	#
	@staticmethod
	def parse(data):
		if isinstance(data, int):
			return EnumLogLevel.__dict__["_value2member_map_"][data]
		elif isinstance(data, str):
			if data in EnumLogLevel.__dict__["_member_names_"]:
				return EnumLogLevel.__dict__[data]
		raise Exception("Unrecognized enumeration value: " + str(data))







#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *
from .LogMessageFormatter import *





#
# This logger will broadcast log messages to additional loggers.
#
class ConsoleLogger(AbstractLogger):



	def __init__(self, idCounter = None, parentID = None, indentationLevel = 0, printToStdErr = False, logMsgFormatter = None):
		super().__init__(idCounter)
		self._indentationLevel = indentationLevel
		if parentID is None:
			parentID = self._idCounter.next()
		self._parentLogEntryID = parentID
		self.__logMsgFormatter = DEFAULT_LOG_MESSAGE_FORMATTER if logMsgFormatter is None else logMsgFormatter

		self.__print = self.__eprint if printToStdErr else print
		self.__printToStdErr = printToStdErr
	#



	@staticmethod
	def create(printToStdErr = False, logMsgFormatter = None):
		return ConsoleLogger(printToStdErr = printToStdErr, logMsgFormatter = logMsgFormatter)
	#



	def __eprint(self, *args, **kwargs):
		print(*args, file=sys.stderr, **kwargs)
	#



	def _logi(self, logEntryStruct, bNeedsIndentationLevelAdaption):
		if bNeedsIndentationLevelAdaption:
			logEntryStruct = list(logEntryStruct)
			logEntryStruct[2] = self._indentationLevel
		lineOrLines = self.__logMsgFormatter.format(logEntryStruct)
		if isinstance(lineOrLines, str):
			self.__print(lineOrLines)
		else:
			for line in lineOrLines:
				self.__print(line)
	#



	def _descend(self, logEntryStruct):
		nextID = logEntryStruct[1]
		return ConsoleLogger(self._idCounter, nextID, self._indentationLevel + 1, self.__printToStdErr, self.__logMsgFormatter)
	#



	def __str__(self):
		return "<ConsoleLogger(" + hex(id(self)) + ", indent=" + str(self._indentationLevel) + ",parentID=" + str(self._parentLogEntryID) + ")>"
	#



	def __repr__(self):
		return "<ConsoleLogger(" + hex(id(self)) + ", indent=" + str(self._indentationLevel) + ",parentID=" + str(self._parentLogEntryID) + ")>"
	#



#









from setuptools import setup


def readme():
	with open('README.rst') as f:
		return f.read()


setup(name='jk_logging',
	version='0.2017.10.8.2',
	description='This is a logging framework.',
	author='Jürgen Knauth',
	author_email='pubsrc@binary-overflow.de',
	license='Apache 2.0',
	url='https://github.com/jkpubsrc/python-module-jk-logging',
	download_url='https://github.com/jkpubsrc/python-module-jk-logging/tarball/0.2017.10.8',
	keywords=[
		'debugging',
		'logging'
		],
	packages=[
		'jk_logging'
		],
	install_requires=[
	],
	include_package_data=True,
	classifiers=[
		'Development Status :: 5 - Production/Stable',
		'Programming Language :: Python :: 3.5',
		'License :: OSI Approved :: Apache Software License'
	],
	long_description=readme(),
	zip_safe=False)




#!/usr/bin/env python3
# -*- coding: utf-8 -*-



from ._inst import instantiateLogMsgFormatter, instantiate




class LoggerInstanceManager(object):

	#
	# Initialization method
	#
	# @param		dict cfgs		The configuration describing how to instantiate the loggers
	#
	def __init__(self, cfgs):
		self.__loggerCfgs = cfgs
		self.__loggerCategoryMaps = {}

		self.__loggerCategories = list(cfgs.keys())
		self.__loggerCategories.sort()
		self.__loggerCategories = tuple(self.__loggerCategories)
	#



	#
	# Returns a list of all categories configured.
	#
	@property
	def categories(self):
		return self.__loggerCategories
	#



	def __getLoggCfg(self, loggerCategory, loggerID):
		cfg = self.__loggerCfgs.get(loggerCategory, None)
		if cfg is None:
			raise Exception("No logger configuration specified for category: " + loggerCategory)

		if loggerID is None:
			if "static" in cfg:
				cfg = cfg["static"]
				sStyle = "static"
			else:
				sStyle = cfg.get("style", None)
				if sStyle is None:
					raise Exception("Missing type for logger configuration in category: " + loggerCategory)
				if sStyle != "static":
					raise Exception("Logger configuration for category '" + loggerCategory + "' is not 'static'!")
		else:
			if "dynamic" in cfg:
				cfg = cfg["dynamic"]
				sStyle = "dynamic"
			else:
				sStyle = cfg.get("style", None)
				if sStyle is None:
					raise Exception("Missing type for logger configuration in category: " + loggerCategory)
				if sStyle != "dynamic":
					raise Exception("Logger configuration for category '" + loggerCategory + "' is not 'dynamic'!")

		cfg = cfg.copy()
		if sStyle == "dynamic":
			adaptationMap = {
				"category": loggerCategory,
				"id": loggerID,
			}
		else:
			adaptationMap = {
				"category": loggerCategory,
			}
		cfg = self.____adaptValue(cfg, adaptationMap)

		return cfg
	#



	def ____adaptValue(self, value, adaptationMap):
		if isinstance(value, str):
			return self.____adaptStr(value, adaptationMap)
		elif isinstance(value, (tuple, list)):
			i = 0
			for v in value:
				value[i] = self.____adaptValue(v, adaptationMap)
				i += 1
		elif isinstance(value, dict):
			for key in value:
				v = value[key]
				value[key] = self.____adaptValue(v, adaptationMap)
		return value
	#



	def ____adaptStr(self, value, adaptationMap):
		assert isinstance(value, str)
		assert value != None

		for key in adaptationMap:
			s = "$(" + key + ")"
			while True:
				pos = value.find(s)
				if pos < 0:
					break
				value = value[0:pos] + adaptationMap[key] + value[pos+len(s):]
		return value
	#



	def getCreateLogger(self, loggerCategory, loggerID = None):
		assert isinstance(loggerCategory, str)

		loggerCategoryEntry = self.__loggerCategoryMaps.get(loggerCategory, None)
		if loggerCategoryEntry is None:
			loggerCategoryEntry = [None, {}]
			self.__loggerCategoryMaps[loggerCategory] = loggerCategoryEntry

		if loggerID is None:
			if loggerCategoryEntry[0] != None:
				return loggerCategoryEntry[0]
			else:
				cfg = self.__getLoggCfg(loggerCategory, loggerID)
				loggerCategoryEntry[0] = instantiate(cfg)
				return loggerCategoryEntry[0]
		else:
			loggerID = str(loggerID)
			logger = loggerCategoryEntry[1].get(loggerID, None)
			if logger != None:
				return logger
			else:
				cfg = self.__getLoggCfg(loggerCategory, loggerID)
				logger = instantiate(cfg)
				loggerCategoryEntry[1][loggerID] = logger
				return logger
	#



	def createLogger(self, loggerCategory, loggerID = None):
		assert isinstance(loggerCategory, str)

		loggerCategoryEntry = self.__loggerCategoryMaps.get(loggerCategory, None)
		if loggerCategoryEntry is None:
			loggerCategoryEntry = [None, {}]
			self.__loggerCategoryMaps[loggerCategory] = loggerCategoryEntry

		if loggerID is None:
			if loggerCategoryEntry[0] != None:
				raise Exception("A logger for category " + str(loggerCategory) + " already exists!")
			else:
				cfg = self.__getLoggCfg(loggerCategory, loggerID)
				loggerCategoryEntry[0] = instantiate(cfg)
				return loggerCategoryEntry[0]
		else:
			loggerID = str(loggerID)
			logger = loggerCategoryEntry[1].get(loggerID, None)
			if logger != None:
				raise Exception("A logger for category " + str(loggerCategory) + " with ID " + loggerID + " already exists!")
			else:
				cfg = self.__getLoggCfg(loggerCategory, loggerID)
				logger = instantiate(cfg)
				loggerCategoryEntry[1][loggerID] = logger
				return logger
	#



	def getLogger(self, loggerCategory, loggerID = None):
		assert isinstance(loggerCategory, str)

		loggerCategoryEntry = self.__loggerCategoryMaps.get(loggerCategory, None)
		if loggerCategoryEntry is None:
			return None

		if loggerID is None:
			return loggerCategoryEntry[0]
		else:
			loggerID = str(loggerID)
			return loggerCategoryEntry[1].get(loggerID, None)
	#



	def closeLogger(self, loggerCategory, loggerID):
		assert isinstance(loggerCategory, str)

		loggerCategoryEntry = self.__loggerCategoryMaps.get(loggerCategory, None)
		if loggerCategoryEntry is None:
			raise Exception("No such log category: " + loggerCategory)

		if loggerID is None:
			if loggerCategoryEntry[0] is None:
				raise Exception("There is no logger for category " + str(loggerCategory) + "!")
			else:
				loggerCategoryEntry[0].close()
				loggerCategoryEntry[0] = None
		else:
			loggerID = str(loggerID)
			logger = loggerCategoryEntry[1].get(loggerID, None)
			if logger != None:
				logger.close()
				loggerCategoryEntry[1][loggerID] = None
			else:
				raise Exception("There is no logger for category " + str(loggerCategory) + " with ID " + loggerID + "!")
	#



	def closeAllLoggers(self):
		for loggereCategory in self.__loggerCategoryMaps:
			loggerCategoryEntry = self.__loggerCategoryMaps[loggereCategory]
			if loggerCategoryEntry[0] != None:
				loggerCategoryEntry[0].close()
				loggerCategoryEntry[0] = None
			for loggerID in loggerCategoryEntry[1]:
				logger = loggerCategoryEntry[1][loggerID]
				logger.close()
			loggerCategoryEntry[1].clear()
	#



#











#!/usr/bin/env python3
# -*- coding: utf-8 -*-



import os
import time
import traceback
import sys
import abc

import sh

from .EnumLogLevel import *
from .AbstractLogger import *
from .AbstractLogMessageFormatter import *
from .LogMessageFormatter import *
from .StringListBuffer import *





#
# This logger writes log data to an internal string list.
#
class StringListLogger(AbstractLogger):


	def __init__(self, idCounter = None, parentID = None, indentationLevel = None, f = None, prefix = None, logMsgFormatter = None):
		super().__init__(idCounter)
		if parentID is None:
			parentID = self._idCounter.next()
		else:
			assert isinstance(parentID, int)
		assert isinstance(f, StringListBuffer)
		assert isinstance(indentationLevel, int)
		if prefix != None:
			assert isinstance(prefix, str)
		self.__logMsgFormatter = DEFAULT_LOG_MESSAGE_FORMATTER if logMsgFormatter is None else logMsgFormatter
		assert isinstance(self.__logMsgFormatter, AbstractLogMessageFormatter)

		self.__f = f
		self._indentationLevel = indentationLevel
		self.__prefix = prefix
	#



	def _logi(self, logEntryStruct, bNeedsIndentationLevelAdaption):
		if self.__f.isClosed:
			raise Exception("Logger already closed.")

		if bNeedsIndentationLevelAdaption:
			logEntryStruct = list(logEntryStruct)
			logEntryStruct[2] = self._indentationLevel
		lineOrLines = self.__logMsgFormatter.format(logEntryStruct)
		self.__f.print(None, lineOrLines)
	#



	# TODO: provide a more efficient implementation. currently <c>_logi()</c> will be called through <c>_logiAll()</c> so that not single but multiple tests are performed if the logger is already closed.
	def _logiAll(self, logEntryStructList, bNeedsIndentationLevelAdaption):
		if self.__f.closed:
			raise Exception("Logger already closed.")

		super()._logiAll(logEntryStructList, bNeedsIndentationLevelAdaption)
	#



	def _descend(self, logEntryStruct):
		nextID = logEntryStruct[1]
		return StringListLogger(self._idCounter, nextID, self._indentationLevel + 1,
			self.__f, self.__prefix, self.__logMsgFormatter)
	#



	#
	# Create a new instance of this logger.
	#
	# @param		AbstractLogMessageFormatter logMsgFormatter
	#
	@staticmethod
	def create(logMsgFormatter = None):
		logFile = StringListBuffer()
		return StringListLogger(None, None, 0, logFile, None, logMsgFormatter)
	#



	def close(self):
		self.__f.close()
	#



	def closed(self):
		return self.__f.isClosed
	#



	def isClosed(self):
		return self.__f.isClosed
	#



	def clear(self):
		return self.__f.clear()
	#



	def toStr(self):
		return self.__f.getDataAsStr()
	#



	def __str__(self):
		return self.__f.getDataAsStr()
	#



	def toList(self):
		return self.__f.getDataAsList()
	#













